/*
The Matrix class represents a 3x3 matrix. Its columns are 3 Vector objects.
*/

package geometry;

public class Matrix {
	
	public static final Matrix IDENTITY = // the identity or unit matrix
	 new Matrix(new Vector(1.0f, 0.0f, 0.0f),
	 	 		new Vector(0.0f, 1.0f, 0.0f),
	 	 		new Vector(0.0f, 0.0f, 1.0f));
	
	private final Vector c1, c2, c3; // the 3 columns
									 // once a Matrix object has been created,
									 // it cannot be changed
	
	public Matrix(Vector c1, Vector c2, Vector c3) {
		this.c1 = c1;
		this.c2 = c2;
		this.c3 = c3;
	}
	
	public Vector getC1() {
		return c1;
	}
	
	public Vector getC2() {
		return c2;
	}
	
	public Vector getC3() {
		return c3;
	}
	
	// method to calculate the determinant
	public float determinant() {
		float r1c1, r1c2, r1c3, r2c1, r2c2, r2c3, r3c1, r3c2, r3c3;
		// these are the elements per row (r) and column (c);
		
		r1c1 = c1.getX();
		r1c2 = c2.getX();
		r1c3 = c3.getX();
		r2c1 = c1.getY();
		r2c2 = c2.getY();
		r2c3 = c3.getY();
		r3c1 = c1.getZ();
		r3c2 = c2.getZ();
		r3c3 = c3.getZ();
		
		float result = r1c1 * (r2c2 * r3c3 - r2c3 * r3c2) +
					   r2c1 * (r3c2 * r1c3 - r3c3 * r1c2) +
					   r3c1 * (r1c2 * r2c3 - r1c3 * r2c2);
		return result;
	}
	
	// method to calculate the trace
	public float trace() {
		float result = c1.getX() + c2.getY() + c3.getZ();
		return result;
	}
	
	// method to calculate the transpose
	public Matrix transpose() {
		Matrix result =
		 new Matrix(new Vector(c1.getX(), c2.getX(), c3.getX()),
		 	 		new Vector(c1.getY(), c2.getY(), c3.getY()),
		 	 		new Vector(c1.getZ(), c2.getZ(), c3.getZ()));
		
		return result;
	}
	
	// method to calculate the multiplication of this matrix with another
	// matrix
	// if this matrix is A and the other matrix is B then the result is AB
	public Matrix multiply(Matrix b) {
		
		// determine the first column of the result matrix
		Vector c = b.getC1();
		Vector result_c1 = new Vector(
		 c1.getX() * c.getX() + c2.getX() * c.getY() + c3.getX() * c.getZ(),
		 c1.getY() * c.getX() + c2.getY() * c.getY() + c3.getY() * c.getZ(),
		 c1.getZ() * c.getX() + c2.getZ() * c.getY() + c3.getZ() * c.getZ()
		 );
		
		// determine the second column of the result matrix
		c = b.getC2();
		Vector result_c2 = new Vector(
		 c1.getX() * c.getX() + c2.getX() * c.getY() + c3.getX() * c.getZ(),
		 c1.getY() * c.getX() + c2.getY() * c.getY() + c3.getY() * c.getZ(),
		 c1.getZ() * c.getX() + c2.getZ() * c.getY() + c3.getZ() * c.getZ()
		 );
		
		// determine the third column of the result matrix
		c = b.getC3();
		Vector result_c3 = new Vector(
		 c1.getX() * c.getX() + c2.getX() * c.getY() + c3.getX() * c.getZ(),
		 c1.getY() * c.getX() + c2.getY() * c.getY() + c3.getY() * c.getZ(),
		 c1.getZ() * c.getX() + c2.getZ() * c.getY() + c3.getZ() * c.getZ()
		 );
		
		// combine the three columns
		Matrix result = new Matrix(result_c1, result_c2, result_c3);
		
		return result;
	}
	
	// method to calculate the addition of a matrix to this matrix
	public Matrix add(Matrix b) {
		
		// determine the three columns of the result matrix
		Vector result_c1 = c1.add(b.getC1());
		Vector result_c2 = c2.add(b.getC2());
		Vector result_c3 = c3.add(b.getC3());
		
		// combine the three columns
		Matrix result = new Matrix(result_c1, result_c2, result_c3);
		
		return result;
	}
	
	public void printMatrix() {
		System.out.println(
		 "[ " + c1.getX() + "  " + c2.getX() + "  " + c3.getX() + "\n" +
		 "  " + c1.getY() + "  " + c2.getY() + "  " + c3.getY() + "\n" +
		 "  " + c1.getZ() + "  " + c2.getZ() + "  " + c3.getZ() + " ]");
	}
	
}
