/*
The PlanarVector class represents a point the the two dimensional space.
It extends from the Vector class and just sets the z coordinate to 0 (but it
does not really matter what this value is).
*/

package geometry;

public class PlanarVector extends Vector {
	
	public PlanarVector(float x, float y) {
		super(x, y, 0.0f);
	}
	
	// convenience method to calculate the intersection point of two lines
	// a + lambda b and c + mu d with a, b, c and d planar vectors
	// the intersection point is returned as the planar vector (lambda, mu)^T
	// note that b and d must be linearly independent!
	public static PlanarVector calculateIntersection(PlanarVector a,
													 PlanarVector b,
													 PlanarVector c,
													 PlanarVector d) {
		// we use Cramer's rule to calulcate lambda and mu
		float det = b.getY() * d.getX() - b.getX() * d.getY();
		float det_lambda = (c.getY()-a.getY()) * d.getX() - (c.getX()-a.getX()) * d.getY();
		float det_mu = (c.getY()-a.getY()) * b.getX() - (c.getX()-a.getX()) * b.getY();
		PlanarVector intersection = new PlanarVector(det_lambda / det, det_mu / det);
		return intersection;
	}
	
	public void printVector() {
		System.out.println("(" + x + ", " + y + ")^T");
	}
	
}
