/*
The Polygon class represents a polygon in the three dimenstional space.
Its vertices are represented by T objects where T is a superclass of Vector.
*/

package geometry;

import java.util.ArrayList;
import display.Screen;

public class Polygon<T extends Vector> {
	
	public static final float ZETA = 0.00001f; // clutter tolerance
	
	protected ArrayList<T> vertices;
	protected float perimeter;
	
	public Polygon(ArrayList<T> vertices) {
		this.vertices = vertices;
		perimeter = calculatePerimeter();
	}
	
	public ArrayList<T> getVertices() {
		return vertices;
	}
	
	public float getPerimeter() {
		return perimeter;
	}
	
	private float calculatePerimeter() {
		float perimeter = 0.0f;
		if (vertices.size() >= 2) {
			for (int i=0; i < vertices.size() - 1; i++) {
				perimeter += vertices.get(i+1).subtract(vertices.get(i)).length();
			}
			perimeter += vertices.get(0).subtract(vertices.get(vertices.size()-1)).length();
		}
		return perimeter;
	}
	
	protected void declutter() {
		// traverse forward
		for (int i=0; i < vertices.size()-1;) {
			if (vertices.get(i+1).subtract(vertices.get(i)).length()
			 > ZETA * perimeter) {
				// vertix stays
				i++;
			} else {
				// vertix is removed
				vertices.remove(i+1);
			}
		}
		// traverse backward
		for (int i=vertices.size()-1; i > 0;) {
			if (vertices.get(0).subtract(vertices.get(vertices.size()-1)).length()
			 > ZETA * perimeter) {
				break;
			} else {
				vertices.remove(vertices.size()-1);
				i--;
			}
		}
	}
	
	// method for calculating the 2D planar representation for this polygon
	// prec-condition: the polygon is on the screen plane
	public PlanarPolygon calculatePlanarRepresentation(Screen screen) {
		ArrayList<PlanarVector> as = new ArrayList<PlanarVector>();
		for(T vertix : vertices) {
			PlanarVector a = vertix.calculatePlanarRepresentation(screen);
			as.add(a);
		}
		PlanarPolygon pp = new PlanarPolygon(as);
		pp.declutter();
		return pp;
	}
	
}
