/*
The Vector class represents a point in the three dimensional space.
*/

package geometry;

import display.Screen;

public class Vector {
	
	protected final float x, y, z; // once a Vector object has been created, it
								 // cannot be changed
	
	
	public Vector(float x, float y, float z) {
		this.x = x;
		this.y = y;
		this.z = z;
	}
	
	public Vector(Vector p) {
		x = p.getX();
		y = p.getY();
		z = p.getZ();
	}
	
	public float getX() {
		return x;
	}
	
	public float getY() {
		return y;
	}
	
	public float getZ() {
		return z;
	}
	
	// method to calclulate the multiplication of this vector with a scalar
	public Vector multiply(float s) {
		Vector result = new Vector(s*x, s*y, s*z);
		return result;
	}

	// method to calculate the addition of a vector to this vector
	public Vector add(Vector p) {
		Vector result = new Vector(x+p.getX(), y+p.getY(), z+p.getZ());
		return result;
	}
	
	// method to calculate the subtraction a vector from this vector
	public Vector subtract(Vector p) {
		Vector result = new Vector(x-p.getX(), y-p.getY(), z-p.getZ());
		return result;
	}
	
	// method to calculate the length of this vector
	public float length() {
		float result = (float)Math.sqrt(x*x + y*y + z*z);
		return result;
	}
	
	// method to calculate the normalization of this vector
	public Vector normalize() {
		float length = length();
		Vector result = new Vector(x/length, y/length, z/length);
		return result;
	}
	
	// inner product between this vector and another vector
	public float innerProduct(Vector p) {
		float result = x * p.getX() + y * p.getY() + z * p.getZ();
		return result;
	}
	
	// method to calculate the cross product between this vector and another
	// vector
	public Vector crossProduct(Vector p) {
		Vector result = new Vector(y * p.getZ() - z * p.getY(),
								   z * p.getX() - x * p.getZ(),
								   x * p.getY() - y * p.getX());
		return result;
	}
	
	// method for calculating the 2D planar representation for this vector
	// pre-condition: the vector is on the screen plane
	public PlanarVector calculatePlanarRepresentation(Screen screen) {
		float x = this.subtract(screen.screenMiddle).innerProduct(screen.u);
		float y = this.subtract(screen.screenMiddle).innerProduct(screen.v);
		PlanarVector a = new PlanarVector(x, y);
		return a;
	}
	
	public void printVector() {
		System.out.println("(" + x + ", " + y + ", " + z + ")^T");
	}
	
}
