/*
This dialog is used for the small window that appears when the user selects
the About menu item in the Help menu.
The dialog's owner is the mainFrame object from the MainWindow class.
The dialog is modal, which means it blocks input to its owner until it is closed.
*/

package gui;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import javax.imageio.ImageIO;

class AboutDialog extends Dialog { // note that his class has default access,
								   // it will be accessed from the MainWindow
								   // class which is in the same package
	
	// constant for general property
	
	public static String VERSION = "6.1"; // the version of the software
	
	// inner class LogoCanvas subclasses the Canvas class,
	// this class is used to draw The Tree Project icon
	private class LogoCanvas extends Canvas {
		private BufferedImage img;
		private LogoCanvas() {
			setPreferredSize(new Dimension(64, 64));
			try {
				img = ImageIO.read(new File("images/tree.png"));
			} catch (IOException e) {
				System.err.println("WARNING: could not read The Tree Project logo");
			}
		}
		public void paint(Graphics g) {
			g.drawImage(img, 0, 0, null);
		}
	}
	
	// the constructor takes the owner (the mainFrame object) as its argument
	public AboutDialog(Frame owner) {
		super(owner, "About The Tree Project", true);
		
		// the dialog has BorderLayout by default;
		// three panels are added to the regions indicated:
		// WEST: the panel which contains The Tree Project logo;
		// CENTER: the panel which contains the text to inform the user;
		// SOUTH: the panel which contains the 'OK' button to close the dialog
		
		Panel logoPanel = new Panel(); // default layout FlowLayout
		LogoCanvas logoCanvas = new LogoCanvas();
		
		Panel textPanel = new Panel(); // default layout FlowLayout but...
		textPanel.setLayout(new GridLayout(2,1)); // ...change to GridLayout
		Label line1 = new Label("The Tree Project | A study in 3D",
		 Label.CENTER);
		Label line2 = new Label("version " + VERSION +
		 ", by Pim Meij   www.jogja.nl", Label.CENTER);
		
		Panel closeDialogPanel = new Panel(); // default layout FlowLayout
		Button oKButton = new Button("OK");
		// enable the 'OK' button to close the dialog window
		oKButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dispose(); // releases all screen resources used by this dialog
			}
		});
		
		logoPanel.add(logoCanvas);
		textPanel.add(line1);
		textPanel.add(line2);
		closeDialogPanel.add(oKButton);
		
		add(logoPanel, BorderLayout.WEST);
		add(textPanel, BorderLayout.CENTER);
		add(closeDialogPanel, BorderLayout.SOUTH);
		
		// for the text: change font style and size
		Font f = line1.getFont();
		Font fd = f.deriveFont(Font.BOLD, 1.5f * f.getSize2D());
		line1.setFont(fd);
		f = line2.getFont();
		fd = f.deriveFont(1.2f * f.getSize2D());
		line2.setFont(fd);
		
		// enable the close window button
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				dispose(); // releases all screen resources used by this dialog
			}
		});
		
		// finally, pack and show
		pack();
		setLocationByPlatform(true); // let the native windowing system decide
									 // where to put the dialog
		setVisible(true);
	}
	
}
