/*
This class will be instantiated when the user starts an animation and will
contail all GUI elements of the animation:
	- animationFrame, the top level window, to which will be added:
		- treeCanvas where our 3D drawing will be done.

When the user closes the top level window and then again starts an animation,
this class will be instantiated again. Buf if the user starts an animation
with the top level window still open, the old instance will be reused.
*/

package gui;

import java.awt.*;
import java.awt.event.*;
import movement.AnimationRunner;

public class AnimationWindow {
	
	// GUI elements:
	private Frame animationFrame;
	private Canvas treeCanvas;
	
	private AnimationRunner aRunner;
	
	public static boolean topLevelWindowExists;
	
	public AnimationWindow() {
		topLevelWindowExists = true;
		launchFrame();
	}
	
	
	public Canvas getTreeCanvas() {
		return treeCanvas;
	}
	
	
	private void launchFrame() {
		animationFrame = new Frame("Animation - The Tree Project");
		animationFrame.setLayout(new FlowLayout());
		treeCanvas = new TreeCanvas (false); // non-manual mode for this canvas
		
		// create the AnimationRunner object and give it all the information it
		// needs
		aRunner = new AnimationRunner(this);
		
		// put all GUI components in their containers
		animationFrame.add(treeCanvas);
		
		// enable the close window button
		animationFrame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				animationFrame.dispose(); // release all screen resources
				topLevelWindowExists = false;
			}
		});
		
		// finally, pack and show
		animationFrame.pack();
		animationFrame.setLocationByPlatform(true); // let the native windowing
													// system decide where to
													// put the window
		animationFrame.setVisible(true);
		
	}
	
	// method doAnimation() is called from the startAnimation() method of the
	// Tree class to run the animation
	// it creates a new thread by means of the AnimationRunner object because
	// that is where all screen movement is done, and then starts the thread
	public void doAnimation() {
		animationFrame.toFront();
		Thread t = new Thread(aRunner);
		t.start();
	}
	
}
