/*
This dialog is used for the dialog that appears when the user checks the
Control menu item in the Animation menu.
The dialog has no owner so it can function independently from the main window.
*/

package gui;

import tree.Tree;
import visual_aid.Markers;
import visual_aid.Trail;

import java.awt.*;
import java.awt.event.*;

class ControlDialog extends Dialog { // note that his class has default access,
									 // it will be accessed from the
									 // ToggleControlItemListener class which
									 // is in the same package
	
	// the contructor takes the mainWindow object as its argument because it is
	// needed in the 'Show markers in manual mode', 'Show trail in manual
	// mode', 'Set', 'Delete', 'Delete all' and 'enable the close window
	// button' parts of the code below
	public ControlDialog(MainWindow mainWindow) {
		
		super((Frame)null, "Animation control"); // the first argument is null
												 // which indicates this dialog
												 // has no owner
												 // note that a cast is
												 // required because there are
												 // several constructors which
												 // take null and String
												 // arguments
		
		// the dialog's BorderLayout is changed to GridLayout
		// four panels are then added:
		// the first panel has a label and checkboxes which can be used to show
		// or hide the markers in manual mode and animation mode
		// only the manual mode checkbox works at this stage
		// the second panel has a label and checkboxes which can be used to
		// show or hide the trail in manual mode and animation mode
		// only the manual mode checkbox works at this stage
		// the third panel has a label and 3 buttons to set a marker, delete a
		// marker and delete all markers
		// the fourth panel has a label and a button to do the animation
		setLayout(new GridLayout(2,3,0,10));
		
		Panel showMarkersPanel = new Panel(new BorderLayout());
		// default FlowLayout changed to BorderLayout
		Label showMarkersPanelLabel = new Label("Show markers", Label.CENTER);
		Panel showMarkersHelperPanel = new Panel();
		// we need a FlowLayout panel so that the checkboxes in the first panel
		// have some space
		Panel modesMarkersPanel = new Panel(new GridLayout(2,1));
		// nevertheless it is to a GridLayout panel that we add the checkboxes
		Checkbox smimm = new Checkbox("in manual mode",
		 Markers.showMarkersInManualMode); // use the flag from the Markers class
		Checkbox smiam = new Checkbox("in animation mode",
		 Markers.showMarkersInAnimationMode); // use the flag from the Markers class
		
		Panel showTrailPanel = new Panel(new BorderLayout());
		// default FlowLayout changed to BorderLayout
		Label showTrailPanelLabel = new Label("Show trail", Label.CENTER);
		Panel showTrailHelperPanel = new Panel();
		// we need a FlowLayout panel so that the checkboxes in the second
		// panel have some space
		Panel modesTrailPanel = new Panel(new GridLayout(2,1));
		// nevertheless it is to a GridLayout panel that we add the checkboxes
		Checkbox stimm = new Checkbox("in manual mode",
		 Trail.showTrailInManualMode); // use the flag from the Trail class
		Checkbox stiam = new Checkbox("in animation mode",
		 Trail.showTrailInAnimationMode); // use the flag from the Trail class
		
		Panel editPanel = new Panel(new BorderLayout()); // default FlowLayout
														 // changed to
														 // BorderLayout
		Label editPanelLabel = new Label("Edit markers", Label.CENTER);
		Panel editHelperPanel = new Panel(); // we need a FlowLayout panel so
											 // that the buttons in the third
											 // panel have some space
		Panel buttonsEditPanel = new Panel(new GridLayout(3,1));
		// nevertheless it is to a GridLayout panel that we add the buttons
		Button set = new Button("Set");
		Button delete = new Button("Delete");
		Button deleteAll = new Button("Delete all");
		
		Panel animatePanel = new Panel(new BorderLayout());
		// default FlowLayout changed to BorderLayout
		Label animatePanelLabel = new Label("Animate", Label.CENTER);
		Panel buttonsAnimatePanel = new Panel();
		Button start = new Button("Start");
		
		Panel fillerPanel1 = new Panel();
		Panel fillerPanel2 = new Panel();
		// these panels just exist to fill the empty spaces so that
		// the animation panel is positioned where we want it
		
		// register an ItemListener object with the Show markers in manual mode
		// checkbox
		smimm.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				Markers.changeSmimmState(smimm, mainWindow);
			}
		});
		
		// register an ItemListener object with the Show markers in animation
		// mode checkbox
		smiam.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				Markers.changeSmiamState(smiam);
			}
		});
		
		// register an ItemListener object with the Show trail in manual mode
		// checkbox
		stimm.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				Trail.changeStimmState(stimm, mainWindow);
			}
		});
		
		// register an ItemListener object with the Show trail in animation
		// mode checkbox
		stiam.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				Trail.changeStiamState(stiam);
			}
		});
		
		// register an ActionListener object with the Set button
		set.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Markers.setMarker(mainWindow);
			}
		});
		
		// register an ActionListener object with the Delete button
		delete.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Markers.deleteMarker(mainWindow);
			}
		});
		
		// register an ActionListener object with the Delete all button
		deleteAll.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Markers.deleteAllMarkers(mainWindow);
			}
		});
		
		// register an ActionListener object with the Start button
		start.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Tree.startAnimation();
			}
		});
		
		modesMarkersPanel.add(smimm);
		modesMarkersPanel.add(smiam);
		showMarkersHelperPanel.add(modesMarkersPanel);
		
		showMarkersPanel.add(showMarkersPanelLabel, BorderLayout.NORTH);
		showMarkersPanel.add(showMarkersHelperPanel, BorderLayout.CENTER);
		
		modesTrailPanel.add(stimm);
		modesTrailPanel.add(stiam);
		showTrailHelperPanel.add(modesTrailPanel);
		
		showTrailPanel.add(showTrailPanelLabel, BorderLayout.NORTH);
		showTrailPanel.add(showTrailHelperPanel, BorderLayout.CENTER);
		
		buttonsEditPanel.add(set);
		buttonsEditPanel.add(delete);
		buttonsEditPanel.add(deleteAll);
		editHelperPanel.add(buttonsEditPanel);
		
		editPanel.add(editPanelLabel, BorderLayout.NORTH);
		editPanel.add(editHelperPanel, BorderLayout.CENTER);
		
		buttonsAnimatePanel.add(start);
		
		animatePanel.add(animatePanelLabel, BorderLayout.NORTH);
		animatePanel.add(buttonsAnimatePanel, BorderLayout.CENTER);
		
		add(showMarkersPanel);
		add(showTrailPanel);
		add(editPanel);
		add(fillerPanel1);
		add(fillerPanel2);
		add(animatePanel);
		
		
		// enable the close window button
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				mainWindow.getControlMenuItem().setState(false);
				// uncheck the Control menu item in the Animation menu
				// (note: this will NOT generate a GUI event)
				dispose(); // releases all screen resources used by this dialog
			}
		});
		
		// finally, pack and show
		pack();
		setLocationByPlatform(true); // let the native windowing system decide
									 // where to put the dialog
		setVisible(true);
	}
	
}
