/*
This dialog is used for the error message that appears when the Delete button
is clicked in the Animation control dialog but no marker was selected.
The dialog's owner is the controlDialog object (the Animation control) from the
ToggleControlItemListener class.
The dialog is modal, which means it blocks input to its owner until it is closed.
*/

package gui;

import java.awt.*;
import java.awt.event.*;

class DeleteErrorDialog extends Dialog { // note that his class has default
										 // access, it will be accessed from
										 // the MainWindow class which is in
										 // the same package
	
	// the constructor takes the tCItemListener object which has a reference to
	// the controlDialog object for Animation control (which will be the owner)
	public DeleteErrorDialog(ToggleControlItemListener tCItemListener) {
		super(tCItemListener.getControlDialog(), "The Tree Project", true);
		
		// the dialog has BorderLayout by default;
		// two panels are added to the regions indicated:
		// CENTER: the panel which contains the text to inform the user;
		// SOUTH: the panel which contains the 'OK' button to close the dialog
		
		Panel textPanel = new Panel(); // default layout FlowLayout but...
		textPanel.setLayout(new GridLayout(3,1)); // ...change to GridLayout
		Label line1 = new Label("   No marker has been deleted.", Label.LEFT);
		line1.setForeground(new Color(0x2E8DEF));
		Label line2 = new Label("   To delete a marker: first select it by clicking on it,",
		 Label.LEFT);
		Label line3 = new Label("   then use the Delete button to delete it.",
		 Label.LEFT);
		Panel closeDialogPanel = new Panel(); // default layout FlowLayout
		Button oKButton = new Button("OK");
		// enable the 'OK' button to close the dialog window
		oKButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dispose(); // releases all screen resources used by this dialog
			}
		});
		
		textPanel.add(line1);
		textPanel.add(line2);
		textPanel.add(line3);
		closeDialogPanel.add(oKButton);
		
		add(textPanel, BorderLayout.CENTER);
		add(closeDialogPanel, BorderLayout.SOUTH);
		
		// for the text: change font style and size
		Font f = line1.getFont();
		Font fd = f.deriveFont(Font.BOLD, 1.2f * f.getSize2D());
		line1.setFont(fd);
		f = line2.getFont();
		fd = f.deriveFont(1.2f * f.getSize2D());
		line2.setFont(fd);
		f = line3.getFont();
		fd = f.deriveFont(1.2f * f.getSize2D());
		line3.setFont(fd);
		
		// enable the close window button
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				dispose(); // releases all screen resources used by this dialog
			}
		});
		
		// finally, pack and show
		pack();
		setLocationByPlatform(true); // let the native windowing system decide
									 // where to put the dialog
		setVisible(true);
	}
	
}
