/*
A LoadAnimationActionListener object is registered with the "Load..." menu item
and the actionPerformed() method is invoked each time the user chooses that
menu item. The method starts a file dialog and when a file is selected, the
file is passed to the Markers class in the visual_aid package to read the file.
If any parse errors were detected, a warning message is launched.
*/

package gui;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import visual_aid.Markers;

public class LoadAnimationActionListener implements ActionListener {
	
	private Frame parent; // the owner of the file dialog
	private MainWindow mainWindow; // the reference to the main window
	private String activeDirectory = "animations"; // the active directory
												   // keeps track where the
												   // file dialog will look for
												   // files, initially this is
												   // the animations/ directory
												   // from the working
												   // directory (../animations/
												   // from this directory)
	
	// the constructor takes the mainFrame object as its argument,
	// this Frame object will be the owner of the FileDialog in the
	// actionPerformed() method
	public LoadAnimationActionListener(Frame parent, MainWindow mainWindow) {
		this.parent = parent;
		this.mainWindow = mainWindow;
	}
	
	public void actionPerformed(ActionEvent e) {
		FileDialog lAFileDialog = new FileDialog(parent, "Load");
		lAFileDialog.setDirectory(activeDirectory);
		
		lAFileDialog.setVisible(true);
		
		File[] animationFile = lAFileDialog.getFiles();
		if (animationFile.length != 0) {
			// if a file was selected, set the new active directory...
			activeDirectory = lAFileDialog.getDirectory();
			// ...and read the file
			int numberOfErrors = Markers.loadAnimationFile(animationFile[0], mainWindow);
			if (numberOfErrors > 0) {
				new ParseErrorDialog(parent, numberOfErrors, animationFile[0].getName(), false);
			}
		}
	}
	
}
