/*
This dialog is used for the warning message that appears when a tree file is
read or an animation file is loaded but parse errors occurred.
The dialog's owner is the mainFrame object from the MainWindow class.
The dialog is modal, which means it blocks input to its owner until it is
closed.
*/

package gui;

import java.awt.*;
import java.awt.event.*;

class ParseErrorDialog extends Dialog { // note that his class has default
										// access, it will be accessed from the
										// ReadTreeActionListener or
										// LoadAnimationActionListener class
										// which is in the same package
	
	// the constructor takes the owner (the mainFrame object), the number of
	// errors, the tree or animation file's name, and a boolean indicating tree
	// or animation file as its arguments
	public ParseErrorDialog(Frame owner, int numberOfErrors,
	 String fileName, boolean isTreeFile) {
		super(owner, "The Tree Project", true);
		
		// the dialog has BorderLayout by default;
		// two panels are added to the regions indicated:
		// CENTER: the panel which contains the text to inform the user;
		// SOUTH: the panel which contains the 'OK' button to close the dialog
		
		Panel textPanel = new Panel(); // default layout FlowLayout but...
		textPanel.setLayout(new GridLayout(3,1)); // ...change to GridLayout
		String errorOrErrors;
		if (numberOfErrors == 1) {
			errorOrErrors = "error";
		} else {
			errorOrErrors = "errors";
		}
		Label line1 = new Label("   When reading the file " + fileName +
		 ", the parser encountered " + numberOfErrors + " " + errorOrErrors +
		 ".", Label.LEFT);
		line1.setForeground(new Color(0x2E8DEF));
		Label line2, line3;
		if (isTreeFile) {
			line2 = new Label("   Some nodes and/or branches may not have been read.",
			 Label.LEFT);
			line3 = new Label("   Correct the file and use the Open menu item to read it again.",
			 Label.LEFT);
		} else {
			line2 = new Label("   Some markers may not have been read.",
			 Label.LEFT);
			line3 = new Label("   Correct the file and use the Load menu item to read it again.",
			 Label.LEFT);
		}
		Panel closeDialogPanel = new Panel(); // default layout FlowLayout
		Button oKButton = new Button("OK");
		// enable the 'OK' button to close the dialog window
		oKButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dispose(); // releases all screen resources used by this dialog
			}
		});
		
		textPanel.add(line1);
		textPanel.add(line2);
		textPanel.add(line3);
		closeDialogPanel.add(oKButton);
		
		add(textPanel, BorderLayout.CENTER);
		add(closeDialogPanel, BorderLayout.SOUTH);
		
		// for the text: change font style and size
		Font f = line1.getFont();
		Font fd = f.deriveFont(Font.BOLD, 1.2f * f.getSize2D());
		line1.setFont(fd);
		f = line2.getFont();
		fd = f.deriveFont(1.2f * f.getSize2D());
		line2.setFont(fd);
		f = line3.getFont();
		fd = f.deriveFont(1.2f * f.getSize2D());
		line3.setFont(fd);
		
		// enable the close window button
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				dispose(); // releases all screen resources used by this dialog
			}
		});
		
		// finally, pack and show
		pack();
		setLocationByPlatform(true); // let the native windowing system decide
									 // where to put the dialog
		setVisible(true);
	}
	
}
