/*
A ReadTreeActionListener object is registered with the "Open..." menu item and
the actionPerformed() method is invoked each time the user chooses that menu
item. The method starts a file dialog and when a file is selected, the file is
passed to the Tree class in the tree package to read the file. If any parse
errors were detected, a warning message is launched.
*/

package gui;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import tree.Tree;

public class ReadTreeActionListener implements ActionListener {
	
	private Frame parent; // the owner of the file dialog
	private String activeDirectory = "trees"; // the active directory keeps
											  // track where the file dialog
											  // will look for files, initially
											  // this is the trees/ directory
											  // from the working directory
											  // (../trees/ from this directory)
	
	// the constructor takes the mainFrame object as its argument,
	// this Frame object will be the owner of the FileDialog in the
	// actionPerformed() method
	public ReadTreeActionListener(Frame parent) {
		this.parent = parent;
	}
	
	public void actionPerformed(ActionEvent e) {
		FileDialog rTFileDialog = new FileDialog(parent, "Open");
		rTFileDialog.setDirectory(activeDirectory);
		
		rTFileDialog.setVisible(true);
		
		File[] treeFile = rTFileDialog.getFiles();
		if (treeFile.length != 0) {
			// if a file was selected, set the new active directory...
			activeDirectory = rTFileDialog.getDirectory();
			// ...and read the file
			int numberOfErrors = Tree.readTreeFile(treeFile[0]);
			if (numberOfErrors > 0) {
				new ParseErrorDialog(parent, numberOfErrors, treeFile[0].getName(), true);
			}
		}
	}
	
}
