/*
A SaveAnimationActionListener object is registered with the "Save..." menu item
and the actionPerformed() method is invoked each time the user chooses that
menu item. The method starts a file dialog and when a new or an existing file
is selected, the file is passed to the Trail class in the visual_aid package to
create the file (in the case of a new file) or overwrite the file (in the case
of an existing file) and save the animation to the file.
*/

package gui;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import visual_aid.Markers;

public class SaveAnimationActionListener implements ActionListener {
	
	private Frame parent; // the owner of the file dialog
	private String activeDirectory = "animations"; // the active directory
												   // keeps track where the
												   // file dialog will propose
												   // to save the file,
												   // initially this is the
												   // animations/ directory
												   // from the working
												   // directory (../animations/
												   // from this directory)
	
	// the constructor takes the mainFrame object as its argument,
	// this Frame object will be the owner of the FileDialog in the
	// actionPerformed() method
	public SaveAnimationActionListener(Frame parent) {
		this.parent = parent;
	}
	
	public void actionPerformed(ActionEvent e) {
		FileDialog sAFileDialog = new FileDialog(parent, "Save", FileDialog.SAVE);
		sAFileDialog.setDirectory(activeDirectory);
		sAFileDialog.setFile("*.taf");
		
		sAFileDialog.setVisible(true);
		
		File[] animationFile = sAFileDialog.getFiles();
		if (animationFile.length != 0) {
			// if a file was selected, set the new active directory...
			activeDirectory = sAFileDialog.getDirectory();
			// ...and save the animation
			Markers.saveAnimationFile(animationFile[0]);
		}
	}
	
}
