/*
A SelectMarkerMouseAdapter object is registered with the canvas and the
mouseClicked() method is invoked each time the user clicks the mouse in the
canvas. The method iterates through the set of markers to examine if a marker
was selected. If no such marker is found, a marker which was selected before
entering this method will become unselected.
*/

package gui;

import java.awt.event.*;
import visual_aid.Marker;
import visual_aid.Markers;

public class SelectMarkerMouseAdapter extends MouseAdapter {
	
	private MainWindow mainWindow;
	
	public SelectMarkerMouseAdapter(MainWindow mainWindow) {
		this.mainWindow = mainWindow; // tell the instance object which window to use
	}
	
	public void mouseClicked(MouseEvent me) {
		Markers markers = Markers.getInstance();
		Markers.selectedMarker = null; // if no marker is found, this value
									   // value will still be null at the end
		
		for (Marker marker : markers) {
			if (marker.getHasScreenRepresentation()) { // only examine markers
													   // which have a screen
													   // representation
				if ((me.getX() >= marker.getX1() - marker.getScreenSize()) &
					(me.getX() <= marker.getX1() + marker.getScreenSize()) &
					(me.getY() >= marker.getY1() - marker.getScreenSize()) &
					(me.getY() <= marker.getY1() + marker.getScreenSize())) {
					// all conditions fulfilled: found marker!
					Markers.selectedMarker = marker;
					break;
				}
			}
		}
		mainWindow.getTreeCanvas().repaint(); // repaint because something may
											  // have changed
	}
}
