/*
A ScreenControlMouseAdapter object is registered with the screen control
buttons (the directions) to start receiving events.

*/

package movement;

import java.awt.event.*;

public class ScreenControlMouseAdapter extends MouseAdapter {
	
	private Thread t;					  // the separate thread
										  // that does the actual movements
	private ScreenControlRunner sCRunner; // the ScreenControlRunner object
										  // which runs the separate thread
	
	public ScreenControlMouseAdapter(ScreenControlRunner sCRunner) {
		this.sCRunner = sCRunner;
	}
	
	public void mousePressed(MouseEvent me) {
		if (me.getButton() == MouseEvent.BUTTON1) { // only react to left mouse button
			if ((t == null) || (!t.isAlive())) { // use || here so the second condition is only checked if t != null
				t = new Thread(sCRunner);
				sCRunner.setSCBObject(me.getSource()); // before starting the thread, pass the button object reference
				t.start();
			}
		}
	}
	
	public void mouseReleased(MouseEvent me) {
		if (me.getButton() == MouseEvent.BUTTON1) { // only react to left mouse button
			if ((t != null) && (t.isAlive())) { // use && here so the second condition is only checked if t != null
				sCRunner.stopRunning(); // stop the ScreenControlRunner thread
			}
		}
	}
	
}
