/*
A ScreenControlRunner object will run a separate thread as long as the screen
is moved in some direction. This thread performs the actual move.
*/

package movement;

import geometry.Vector;
import gui.MainWindow;
import java.util.HashMap;

import static display.Screen.MAIN_SCREEN;
import static display.Screen.RETINA_SCREEN_DISTANCE;

public class ScreenControlRunner implements Runnable {
	
	private final int SLEEP_TIME = 100;
	private int sleepTime = SLEEP_TIME;
	
	private final float SHIFT = 10.0f;
	private float shift = SHIFT;
	
	private final float SHIFT_ACCELERATOR_1 = 1.7f;
	private float shiftAccelerator1 = SHIFT_ACCELERATOR_1;
	
	private final int SHIFT_ACCELERATOR_1_MAX_NUMBER = 6;
	private int shiftAccelerator1MaxNumber = SHIFT_ACCELERATOR_1_MAX_NUMBER;
	
	private final float SHIFT_ACCELERATOR_2 = 1.3f;
	private float shiftAccelerator2 = SHIFT_ACCELERATOR_2;
	
	private final int SHIFT_ACCELERATOR_2_MAX_NUMBER = 20;
	private int shiftAccelerator2MaxNumber = SHIFT_ACCELERATOR_2_MAX_NUMBER;
	
	private final float ROTATE = (float)Math.PI/180.0f;
	private float rotate = ROTATE;
	
	private final float ROTATE_ACCELERATOR_1 = 1.0f;
	private float rotateAccelerator1 = ROTATE_ACCELERATOR_1;
	
	private final int ROTATE_ACCELERATOR_1_MAX_NUMBER = 4;
	private int rotateAccelerator1MaxNumber = ROTATE_ACCELERATOR_1_MAX_NUMBER;
	
	private final float ROTATE_ACCELERATOR_2 = 1.2f;
	private float rotateAccelerator2 = ROTATE_ACCELERATOR_2;
	
	private final int ROTATE_ACCELERATOR_2_MAX_NUMBER = 18;
	private int rotateAccelerator2MaxNumber = ROTATE_ACCELERATOR_2_MAX_NUMBER;
	
	
	private boolean timeToQuit;
	private MainWindow mainWindow;
	private HashMap<Object, Direction> sCBD;
	
	private Object sCBObject;
	private Direction direction;
	
	public ScreenControlRunner(MainWindow mainWindow, HashMap<Object, Direction> sCBD) {
		this.mainWindow = mainWindow; // tell the instance object which window to use
		this.sCBD = sCBD; // pass the instance object the HashMap that maps
						  // screen control buttons to directions
	}
	
	public void setSCBObject(Object sCBObject) { // this method must be called
												 // before the thread is started
		this.sCBObject = sCBObject;
	}
	
	public void run() {
		timeToQuit = false;
		direction = sCBD.get(sCBObject);
		if (direction == null) {
			System.err.println("FATAL: button not in HashMap sCBD in ScreenControlRunner");
		}
		int shiftCounter = 1;
		int rotateCounter = 1;
		float variableShift = shift;
		float variableRotate = rotate;
		Vector ux, uy, u, vy, vz, v, normalz, normalx, normal;
		
		while (!timeToQuit) {
			switch (direction) {
			case X_SHIFT_POSITIVE:
				// first: calculate the new screen middle and retina
				MAIN_SCREEN.screenMiddle =
				 MAIN_SCREEN.screenMiddle.add(MAIN_SCREEN.u.multiply(variableShift));
				MAIN_SCREEN.retina =
				 MAIN_SCREEN.screenMiddle.subtract(MAIN_SCREEN.normal.multiply(RETINA_SCREEN_DISTANCE));
				shiftCounter++;
				if (shiftCounter <= shiftAccelerator1MaxNumber) {
					variableShift = shiftAccelerator1 * variableShift;
				} else if (shiftCounter <= shiftAccelerator2MaxNumber) {
					variableShift = shiftAccelerator2 * variableShift;
				}
				break;
			case X_SHIFT_NEGATIVE:
				// first: calculate the new screen middle and retina
				MAIN_SCREEN.screenMiddle =
				 MAIN_SCREEN.screenMiddle.subtract(MAIN_SCREEN.u.multiply(variableShift));
				MAIN_SCREEN.retina =
				 MAIN_SCREEN.screenMiddle.subtract(MAIN_SCREEN.normal.multiply(RETINA_SCREEN_DISTANCE));
				shiftCounter++;
				if (shiftCounter <= shiftAccelerator1MaxNumber) {
					variableShift = shiftAccelerator1 * variableShift;
				} else if (shiftCounter <= shiftAccelerator2MaxNumber) {
					variableShift = shiftAccelerator2 * variableShift;
				}
				break;
			case Y_SHIFT_POSITIVE:
				// first: calculate the new screen middle and retina
				MAIN_SCREEN.screenMiddle =
				 MAIN_SCREEN.screenMiddle.add(MAIN_SCREEN.v.multiply(variableShift));
				MAIN_SCREEN.retina =
				 MAIN_SCREEN.screenMiddle.subtract(MAIN_SCREEN.normal.multiply(RETINA_SCREEN_DISTANCE));
				shiftCounter++;
				if (shiftCounter <= shiftAccelerator1MaxNumber) {
					variableShift = shiftAccelerator1 * variableShift;
				} else if (shiftCounter <= shiftAccelerator2MaxNumber) {
					variableShift = shiftAccelerator2 * variableShift;
				}
				break;
			case Y_SHIFT_NEGATIVE:
				// first: calculate the new screen middle and retina
				MAIN_SCREEN.screenMiddle =
				 MAIN_SCREEN.screenMiddle.subtract(MAIN_SCREEN.v.multiply(variableShift));
				MAIN_SCREEN.retina =
				 MAIN_SCREEN.screenMiddle.subtract(MAIN_SCREEN.normal.multiply(RETINA_SCREEN_DISTANCE));
				shiftCounter++;
				if (shiftCounter <= shiftAccelerator1MaxNumber) {
					variableShift = shiftAccelerator1 * variableShift;
				} else if (shiftCounter <= shiftAccelerator2MaxNumber) {
					variableShift = shiftAccelerator2 * variableShift;
				}
				break;
			case Z_SHIFT_POSITIVE:
				// first: calculate the new screen middle and retina
				MAIN_SCREEN.screenMiddle =
				 MAIN_SCREEN.screenMiddle.add(MAIN_SCREEN.normal.multiply(variableShift));
				MAIN_SCREEN.retina =
				 MAIN_SCREEN.screenMiddle.subtract(MAIN_SCREEN.normal.multiply(RETINA_SCREEN_DISTANCE));
				shiftCounter++;
				if (shiftCounter <= shiftAccelerator1MaxNumber) {
					variableShift = shiftAccelerator1 * variableShift;
				} else if (shiftCounter <= shiftAccelerator2MaxNumber) {
					variableShift = shiftAccelerator2 * variableShift;
				}
				break;
			case Z_SHIFT_NEGATIVE:
				// first: calculate the new screen middle and retina
				MAIN_SCREEN.screenMiddle =
				 MAIN_SCREEN.screenMiddle.subtract(MAIN_SCREEN.normal.multiply(variableShift));
				MAIN_SCREEN.retina =
				 MAIN_SCREEN.screenMiddle.subtract(MAIN_SCREEN.normal.multiply(RETINA_SCREEN_DISTANCE));
				shiftCounter++;
				if (shiftCounter <= shiftAccelerator1MaxNumber) {
					variableShift = shiftAccelerator1 * variableShift;
				} else if (shiftCounter <= shiftAccelerator2MaxNumber) {
					variableShift = shiftAccelerator2 * variableShift;
				}
				break;
			case X_ROTATE_Y_TO_Z:
				// first: calculate the new v, n and retina
				vy = MAIN_SCREEN.v.multiply((float)Math.cos(variableRotate));
				vz = MAIN_SCREEN.normal.multiply((float)Math.sin(variableRotate));
				v = vy.add(vz);
				normal = MAIN_SCREEN.u.crossProduct(v).normalize();
				v = normal.crossProduct(MAIN_SCREEN.u).normalize();
				MAIN_SCREEN.v = v;
				MAIN_SCREEN.normal = normal;
				MAIN_SCREEN.retina =
				 MAIN_SCREEN.screenMiddle.subtract(MAIN_SCREEN.normal.multiply(RETINA_SCREEN_DISTANCE));
				rotateCounter++;
				if (rotateCounter <= rotateAccelerator1MaxNumber) {
					variableRotate = rotateAccelerator1 * variableRotate;
				} else if (rotateCounter <= rotateAccelerator2MaxNumber) {
					variableRotate = rotateAccelerator2 * variableRotate;
				}
				break;
			case X_ROTATE_Z_TO_Y:
				// first: calculate the new v, n and retina
				vy = MAIN_SCREEN.v.multiply((float)Math.cos(variableRotate));
				vz = MAIN_SCREEN.normal.multiply((float)Math.sin(variableRotate));
				v = vy.subtract(vz);
				normal = MAIN_SCREEN.u.crossProduct(v).normalize();
				v = normal.crossProduct(MAIN_SCREEN.u).normalize();
				MAIN_SCREEN.v = v;
				MAIN_SCREEN.normal = normal;
				MAIN_SCREEN.retina =
				 MAIN_SCREEN.screenMiddle.subtract(MAIN_SCREEN.normal.multiply(RETINA_SCREEN_DISTANCE));
				rotateCounter++;
				if (rotateCounter <= rotateAccelerator1MaxNumber) {
					variableRotate = rotateAccelerator1 * variableRotate;
				} else if (rotateCounter <= rotateAccelerator2MaxNumber) {
					variableRotate = rotateAccelerator2 * variableRotate;
				}
				break;
			case Y_ROTATE_Z_TO_X:
				// first: calculate the new n, u and retina
				normalz = MAIN_SCREEN.normal.multiply((float)Math.cos(variableRotate));
				normalx = MAIN_SCREEN.u.multiply((float)Math.sin(variableRotate));
				normal = normalz.add(normalx);
				u = MAIN_SCREEN.v.crossProduct(normal).normalize();
				normal = u.crossProduct(MAIN_SCREEN.v).normalize();
				MAIN_SCREEN.normal = normal;
				MAIN_SCREEN.u = u;
				MAIN_SCREEN.retina =
				 MAIN_SCREEN.screenMiddle.subtract(MAIN_SCREEN.normal.multiply(RETINA_SCREEN_DISTANCE));
				rotateCounter++;
				if (rotateCounter <= rotateAccelerator1MaxNumber) {
					variableRotate = rotateAccelerator1 * variableRotate;
				} else if (rotateCounter <= rotateAccelerator2MaxNumber) {
					variableRotate = rotateAccelerator2 * variableRotate;
				}
				break;
			case Y_ROTATE_X_TO_Z:
				// first: calculate the new n, u and retina
				normalz = MAIN_SCREEN.normal.multiply((float)Math.cos(variableRotate));
				normalx = MAIN_SCREEN.u.multiply((float)Math.sin(variableRotate));
				normal = normalz.subtract(normalx);
				u = MAIN_SCREEN.v.crossProduct(normal).normalize();
				normal = u.crossProduct(MAIN_SCREEN.v).normalize();
				MAIN_SCREEN.normal = normal;
				MAIN_SCREEN.u = u;
				MAIN_SCREEN.retina =
				 MAIN_SCREEN.screenMiddle.subtract(MAIN_SCREEN.normal.multiply(RETINA_SCREEN_DISTANCE));
				rotateCounter++;
				if (rotateCounter <= rotateAccelerator1MaxNumber) {
					variableRotate = rotateAccelerator1 * variableRotate;
				} else if (rotateCounter <= rotateAccelerator2MaxNumber) {
					variableRotate = rotateAccelerator2 * variableRotate;
				}
				break;
			case Z_ROTATE_X_TO_Y:
				// first: calculate the new u and v (the retina stays the same)
				ux = MAIN_SCREEN.u.multiply((float)Math.cos(variableRotate));
				uy = MAIN_SCREEN.v.multiply((float)Math.sin(variableRotate));
				u = ux.add(uy);
				v = MAIN_SCREEN.normal.crossProduct(u).normalize();
				u = v.crossProduct(MAIN_SCREEN.normal).normalize();
				MAIN_SCREEN.u = u;
				MAIN_SCREEN.v = v;
				rotateCounter++;
				if (rotateCounter <= rotateAccelerator1MaxNumber) {
					variableRotate = rotateAccelerator1 * variableRotate;
				} else if (rotateCounter <= rotateAccelerator2MaxNumber) {
					variableRotate = rotateAccelerator2 * variableRotate;
				}
				break;
			case Z_ROTATE_Y_TO_X:
				// first: calculate the new u and v (the retina stays the same)
				ux = MAIN_SCREEN.u.multiply((float)Math.cos(variableRotate));
				uy = MAIN_SCREEN.v.multiply((float)Math.sin(variableRotate));
				u = ux.subtract(uy);
				v = MAIN_SCREEN.normal.crossProduct(u).normalize();
				u = v.crossProduct(MAIN_SCREEN.normal).normalize();
				MAIN_SCREEN.u = u;
				MAIN_SCREEN.v = v;
				rotateCounter++;
				if (rotateCounter <= rotateAccelerator1MaxNumber) {
					variableRotate = rotateAccelerator1 * variableRotate;
				} else if (rotateCounter <= rotateAccelerator2MaxNumber) {
					variableRotate = rotateAccelerator2 * variableRotate;
				}
				break;
			default:
				System.err.println("ERROR: screen control not handled in ScreenControlRunner");
				// this "case" should never happen
			}
			
			// second: find the screen representation
			MAIN_SCREEN.findRepresentation();
			
			// third: repaint
			// (this part is the same for all possible directions)
			mainWindow.getTreeCanvas().repaint();
			
			// fourth:
			// note that the AWT GUI thread keeps running and has time to deal
			// with requests to update() and paint() because the thread which
			// is the source of the requests (this thread) is put to sleep
			// periodically and thus slowed down, the AWT thread meanwhile goes
			// on its merry way
			//
			// my thanks go to the author of this article:
			// http://www.scs.ryerson.ca/~mes/courses/cps530/programs/threads/Repaint/index.html
			try {
				Thread.sleep(sleepTime);
			} catch (InterruptedException e) {
				System.err.println("WARNING: thread had its sleep interrupted in ScreenControlRunner");
			}
			
			
		}
		
	}
	
	public void stopRunning() {
		timeToQuit = true;
	}
	
	// the ScreenControlRunner ojbect also runs the short animation to welcome
	// the user
	public void doAnimation() {
		// a pyramid is already visible and screen middle is at
		// (0, 2000, -5000)^T, the screen spirals twice around the pyramids and
		// ends with screen middle at (0, 0, 0)^T
		
		for (int i=99; i>=0; i--) { // perform 100 steps
			
			// with a 10 ms pause between two steps
			try {
				Thread.sleep(10);
			} catch (InterruptedException e) {
				System.err.println("WARNING: thread had its sleep interrupted in ScreenControlRunner");
			}
			
			float angle = (float)i/100.0f *4.0f*(float)Math.PI;
			float cosAngle = (float)Math.cos(angle);
			float sinAngle = (float)Math.sin(angle);
			
			// determine new screen middle
			float x = - (1250.0f + (float)i/100.0f * 5000.0f) * sinAngle;
			float y = (float)i/100.0f * 2000.0f;
			float z = 1250.0f - (1250.0f + (float)i/100.0f * 5000.0f) * cosAngle;
			MAIN_SCREEN.screenMiddle = new Vector(x, y, z);
			
			// determine new u and n (v does not change)
			MAIN_SCREEN.u = new Vector(cosAngle, 0.0f, -sinAngle);
			MAIN_SCREEN.normal =new Vector(sinAngle, 0.0f, cosAngle);
			
			// calculate new retina
			MAIN_SCREEN.retina =
			 MAIN_SCREEN.screenMiddle.subtract(MAIN_SCREEN.normal.multiply(RETINA_SCREEN_DISTANCE));
			
			// find the screen representation
			MAIN_SCREEN.findRepresentation();
			
			// repaint
			mainWindow.getTreeCanvas().repaint();
		}
	}
	
	
}
