/*
A tree is constructed of nodes and branches.
A node is a point somewhere in the three dimensional space.
A branch is a straight line between two nodes.
*/

package tree;

import display.Screen;
import geometry.Matrix;
import geometry.Vector;

public class Node extends Vector {
	
	public Node(float x, float y, float z) {
		super(x, y, z);
	}
	
	public Node(Vector p) {
		super(p);
	}
	
	// method to determine if the node is visible for a given screen
	// i.e. the node is on the visible side of the retina plane
	public boolean isVisible(Screen screen) {
		float innerProduct =
		 this.subtract(screen.retina).innerProduct(screen.normal);
		return (innerProduct > 0.0f);
	}
	
	
	// method for projecting the node (p) on the given screen
	// pre-condition: the node is visible
	public Vector projectOnScreen(Screen screen) {
		// see the Screen class
		// we are determining lambda such that
		// p2Prime = p + lambda (r - p) = s^m + mu u + nu v
		// or [ p - r  u  v ] (lambda, mu, nu)^T = p - s^m
		// we use Cramer's rule to calculate lambda
		Matrix M = new Matrix(this.subtract(screen.retina),
		 screen.u, screen.v);
		Matrix M_lambda = new Matrix(this.subtract(screen.screenMiddle),
		 screen.u, screen.v);
		float lambda = M_lambda.determinant() / M.determinant();
		Vector p2Prime = new Vector(this.add(screen.retina.subtract(this).multiply(lambda)));
		return p2Prime;
	}
	
}
