/*
This enumeration is used for the parser states in the use case "Parse tree
file".

Eleven values are defined as follows:

	- IGNORE_INPUT: the next token should be [NODES], [BRANCHES] or [SURFACE],
	  other tokens will be ignored i.e. do not contribute to the tree;
	- READ_NODE_NAME: the next token should be a string and it will be the name
	  of a new node;
	- READ_NODE_X: the next token should be a float and it will be the x
	  coordinate of the node whose name has just been read;
	- READ_NODE_Y: the next token should be a float and it will be the y
	  coordinate of the node whose name and x coordinate have just been read;
	- READ_NODE_Z: the next token should be a float and it will be the z
	  coordinate of the node whose name and x and y coordinates have just been
	  read;
	- READ_BRANCH_NAME: the next token should be a string and it will be the
	  name of a new branch;
	- READ_BRANCH_P: the next token should be a string and the name of a node
	  that has been read and it will be the p node of the branch whose name has
	  just been read;
	- READ_BRANCH_Q: the next token should be a string and the name of a node
	  that has been read and it will be the q node of the branch whose name and
	  p node have just been read;
	- READ_COLOR: the next token should be an integer between 0 and 16777215
	  and it will be the color of a new surface;
	- READ_ORIENTATION: the next token should be NEGATIVE or POSITIVE and it
	  will be the orientation of the surface under construction;
	- READ_SURFACE_P: the next token should be a string and the name of a node
	  that has been read and it will be the next node of the surface under
	  construction.
	
	Note that for READ_NODE_NAME, READ_BRANCH_NAME and READ_SURFACE_P it is
	also allowed that the next token will be [NODES], [BRANCHES] or [SURFACE]
	or that EOF has been reached.
	
*/

package tree;

public enum ParseState {
	IGNORE_INPUT,
	READ_NODE_NAME,
	READ_NODE_X,
	READ_NODE_Y,
	READ_NODE_Z,
	READ_BRANCH_NAME,
	READ_BRANCH_P,
	READ_BRANCH_Q,
	READ_COLOR,
	READ_ORIENTATION,
	READ_SURFACE_P
}
