/*
A Trail object is a HashSet of TrailSegment objects.
This class has several static variables which are properties of all trail
segments:
- The flag showTrailInManualMode determines whether trail segments will be
shown in manual mode.
- The flag showTrailInAnimationMode determines whether trail segments will be
shown in animation mode.
*/

package visual_aid;

import gui.MainWindow;

import java.awt.*;
import java.util.ArrayList;

public class Trail extends ArrayList<TrailSegment> {
	
	// flags:
	
	public static final boolean SHOW_TRAIL_IN_MANUAL_MODE = true;
	public static boolean showTrailInManualMode = SHOW_TRAIL_IN_MANUAL_MODE;
	// flags whether trail segments shall be shown during manual screen control
	
	public static final boolean SHOW_TRAIL_IN_ANIMATION_MODE = false;
	public static boolean showTrailInAnimationMode = SHOW_TRAIL_IN_ANIMATION_MODE;
	// flags whether trail segments shall be shown during animated screen control
	
	// constants for general properties
	// constants are read from a file
	// for a constant which can be read from a file, an attempt is made to read
	// a proper value from the file but if this fails then a default value will
	// be used
	
	// constants read from file (and their default values)
	
	private static float TRAIL_SEGMENT_SIZE_DEFAULT = 1.0f; // the size for the trail segments
	public static float TRAIL_SEGMENT_SIZE = TRAIL_SEGMENT_SIZE_DEFAULT;
	
	// Trail is implemented as a singleton class
	private static Trail trail = new Trail();
	
	// A private constructor prevents any other class from instantiating
	private Trail() {}
	
	// A method to get the reference to the singleton class
	public static Trail getInstance() {
		return trail;
	}
	
	// this method is called when the user checks or unchecks "Show trail in
	// manual mode"
	public static void changeStimmState(Checkbox stimm, MainWindow mainWindow) {
		showTrailInManualMode = stimm.getState();
		mainWindow.getTreeCanvas().repaint();
	}
	
	// this method is called when the user checks or unchecks "Show trail in
	// animation mode"
	public static void changeStiamState(Checkbox stiam) {
		showTrailInAnimationMode = stiam.getState();
		// for the rest we do nothing because the change shall only visible
		// once the user starts an animation
	}
	
	// this method is called whenever the markers change: the trail is then
	// regenerated from the list of markers
	public static void generateTrail() {
		trail = new Trail(); // reset the trail object
		Markers markers = Markers.getInstance();
		Marker startMarker = null, endMarker = null; // define two variables
													 // which will hold the
													 // start and end markers
													 // for each trail segment
													 // which will be created
		for (Marker marker : markers) { // iterate through the list of markers
			if (startMarker == null) {
				startMarker = marker; // if we have set neither start nor end
									  // end marker, then set the start marker
			} else {
				if (endMarker == null) {
					endMarker = marker; // if we have set the start marker but
										// not the end marker, then set the end
										// marker
					trail.add(new TrailSegment(startMarker, endMarker));
					// and add a new trail segment to the trail
				} else {
					startMarker = endMarker; // if we have set both start and
											 // end marker, then move the end
											 // marker to the start marker
					endMarker = marker; // set the end marker to its new value
					trail.add(new TrailSegment(startMarker, endMarker));
					// and add a new trail segment to the trail
				}
			}
		}
	}
	
}
